#ifndef __DUNE_MMDG_MMDGPROBLEM1_HH__
#define __DUNE_MMDG_MMDGPROBLEM1_HH__

#include <cmath>
#include <dune/mmdg/problems/coupleddgproblem.hh>

//a coupled dg problem
template<class Coordinate, class Scalar = double>
class MMDGProblem1 : public CoupledDGProblem<Coordinate, Scalar>
{
public:
  static constexpr int dim = Coordinate::dimension;
  using Base = CoupledDGProblem<Coordinate, Scalar>;
  using Matrix = typename Base::Matrix;

  //constructor
  MMDGProblem1 (const Scalar d) : d_(d) {}

  //the exact bulk solution at position pos
  Scalar exactSolution (const Coordinate& pos) const
  {
    return (pos[0] < 0.5) ? pos[0] : pos[0] + 1;
  }

  //the exact solution on the interface at position pos
  Scalar exactInterfaceSolution (const Coordinate& pos) const
  {
    return 1.0;
  }

  //indicates whether an exact solution is implemented for the problem
  bool hasExactSolution () const
  {
    return true;
  };

  //aperture d of the fracture at position pos
  Scalar aperture (const Coordinate& pos) const
  {
    return d_ * (pos[1] + 0.01);
    return d_;
  }

  //interface source term at position pos
  Scalar qInterface (const Coordinate& pos) const
  {
    const Scalar de = aperture(pos);
    const Scalar dPrime2 = gradAperture(pos) * gradAperture(pos);
    const Scalar dPrimePrime = 0.0;

    return (-de * dPrimePrime - dPrime2) * de;

    return Scalar(0.0);
  }

  //tangential gradient of the aperture d of the fracture at position pos
  Coordinate gradAperture (const Coordinate& pos) const
  {
   Coordinate gradD(0.0);
   gradD[1] = d_;
   return gradD;

   return Coordinate(0.0);
  }

  //tangential permeability per aperture tensor of the interface at position pos
  Matrix Kparallel (const Coordinate& pos) const
  {
    Matrix permeability(0.0);

    for (int i = 0; i < dim; i++)
    {
      permeability[i][i] = 1.0 * aperture(pos);
    }

    //return identity matrix
    return permeability;
  }


  //returns the recommended quadrature order to compute an integral
  //over x * boundary(x)
  int quadratureOrderBoundary () const
  {
    return 10;//2;
  }

private:
  Scalar d_; //aperture
};

#endif
