#ifndef __DUNE_MMDG_MMDGPROBLEM4_HH__
#define __DUNE_MMDG_MMDGPROBLEM4_HH__

#include <cmath>
#include <dune/mmdg/problems/coupleddgproblem.hh>

//a coupled dg problem
template<class Coordinate, class Scalar = double>
class MMDGProblem4 : public CoupledDGProblem<Coordinate, Scalar>
{
public:
  static constexpr int dim = Coordinate::dimension;
  static constexpr Scalar jump = 1.0;
  using Base = CoupledDGProblem<Coordinate, Scalar>;
  using Matrix = typename Base::Matrix;

  //constructor
  MMDGProblem4 (const Scalar d) : d_(d) {}

  //the exact bulk solution at position pos
  Scalar exactSolution (const Coordinate& pos) const
  {
    Scalar solution = pos[0] * pos[0] - pos[1] * pos[1];

    if (pos[0] > 0.5)
    {
      return solution += jump;
    }

    return solution;
  }

  //the exact solution on the interface at position pos
  Scalar exactInterfaceSolution (const Coordinate& pos) const
  {
    return 0.25 - pos[1] * pos[1] + 0.5 * jump;
  }

  //indicates whether an exact solution is implemented for the problem
  bool hasExactSolution () const
  {
    return true;
  };

  //interface source term at position pos
  Scalar qInterface (const Coordinate& pos) const
  {
    // return 2.0 * d_ * d_;
    const Scalar de = aperture(pos);
    const Scalar dPrime = sqrt(gradAperture(pos) * gradAperture(pos));
    const Scalar dPrimePrime = 0.;

    return (d_ * d_ * ( 10. * pos[1] * 0.01 + 9. * pos[1] * pos[1] + 2. * 0.01 * 0.01 - 0.5 * (jump + 0.5))) * de;
  }

  //aperture d of the fracture at position pos
  Scalar aperture (const Coordinate& pos) const
  {
    return d_ * (pos[1] + 0.01);;
  }

  //tangential gradient of the aperture d of the fracture at position pos
  Coordinate gradAperture (const Coordinate& pos) const
  {
    Coordinate gradD(0.0);
    gradD[1] = d_;
    return gradD;

    return Coordinate(0.0);
  }

  //permeability of the fracture in normal direction at position pos
  Scalar Kperp (const Coordinate& pos) const
  {
    return Scalar(1.0) / jump;
  }

  //tangential permeability tensor of the interface at position pos
  Matrix Kparallel (const Coordinate& pos) const
  {
    Matrix permeability(0.0);

    for (int i = 0; i < dim; i++)
    {
      permeability[i][i] = 1.0 * aperture(pos);//d_;
    }

    return permeability;
  }

  //returns the recommended quadrature order to compute an integral
  //over x * boundary(x)
  int quadratureOrderBoundary () const
  {
    return 10;//3;
  }

  //returns the recommended quadrature order to compute an integral
  //over  x * d^2 * interfaceBoundary(x)
  //and   x * d * interfaceBoundary(x) * (Kpar * grad(d))
  //and   d^2 * interfaceBoundary(x) * Kpar
  int quadratureOrderInterfaceBoundary () const
  {
    return 10;//3;
  }

private:
  Scalar d_; //aperture

};

#endif
